/* valaarraymovemethod.c generated by valac, the Vala compiler
 * generated from valaarraymovemethod.vala, do not modify */

/* valaarraymovemethod.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "vala.h"
#include <glib.h>

#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))

static gpointer vala_array_move_method_parent_class = NULL;

static GType vala_array_move_method_get_type_once (void);

/**
 * Creates a new array move method.
 *
 * @return newly created method
 */
ValaArrayMoveMethod*
vala_array_move_method_construct (GType object_type,
                                  ValaSourceReference* source_reference)
{
	ValaArrayMoveMethod* self = NULL;
	ValaVoidType* _tmp0_;
	ValaVoidType* _tmp1_;
	_tmp0_ = vala_void_type_new (NULL);
	_tmp1_ = _tmp0_;
	self = (ValaArrayMoveMethod*) vala_method_construct (object_type, "move", (ValaDataType*) _tmp1_, source_reference, NULL);
	_vala_code_node_unref0 (_tmp1_);
	vala_symbol_set_external ((ValaSymbol*) self, TRUE);
	return self;
}

ValaArrayMoveMethod*
vala_array_move_method_new (ValaSourceReference* source_reference)
{
	return vala_array_move_method_construct (VALA_TYPE_ARRAY_MOVE_METHOD, source_reference);
}

static void
vala_array_move_method_class_init (ValaArrayMoveMethodClass * klass,
                                   gpointer klass_data)
{
	vala_array_move_method_parent_class = g_type_class_peek_parent (klass);
}

static void
vala_array_move_method_instance_init (ValaArrayMoveMethod * self,
                                      gpointer klass)
{
}

/**
 * Represents the Array.move method.
 */
static GType
vala_array_move_method_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaArrayMoveMethodClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_array_move_method_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaArrayMoveMethod), 0, (GInstanceInitFunc) vala_array_move_method_instance_init, NULL };
	GType vala_array_move_method_type_id;
	vala_array_move_method_type_id = g_type_register_static (VALA_TYPE_METHOD, "ValaArrayMoveMethod", &g_define_type_info, 0);
	return vala_array_move_method_type_id;
}

GType
vala_array_move_method_get_type (void)
{
	static volatile gsize vala_array_move_method_type_id__once = 0;
	if (g_once_init_enter (&vala_array_move_method_type_id__once)) {
		GType vala_array_move_method_type_id;
		vala_array_move_method_type_id = vala_array_move_method_get_type_once ();
		g_once_init_leave (&vala_array_move_method_type_id__once, vala_array_move_method_type_id);
	}
	return vala_array_move_method_type_id__once;
}

