/* valaccodeifstatement.c generated by valac, the Vala compiler
 * generated from valaccodeifstatement.vala, do not modify */

/* valaccodeifstatement.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valaccode.h"
#include <glib.h>

#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))

struct _ValaCCodeIfStatementPrivate {
	ValaCCodeExpression* _condition;
	ValaCCodeStatement* _true_statement;
	ValaCCodeStatement* _false_statement;
	gboolean _else_if;
};

static gint ValaCCodeIfStatement_private_offset;
static gpointer vala_ccode_if_statement_parent_class = NULL;

static void vala_ccode_if_statement_real_write (ValaCCodeNode* base,
                                         ValaCCodeWriter* writer);
static void vala_ccode_if_statement_finalize (ValaCCodeNode * obj);
static GType vala_ccode_if_statement_get_type_once (void);

static inline gpointer
vala_ccode_if_statement_get_instance_private (ValaCCodeIfStatement* self)
{
	return G_STRUCT_MEMBER_P (self, ValaCCodeIfStatement_private_offset);
}

ValaCCodeExpression*
vala_ccode_if_statement_get_condition (ValaCCodeIfStatement* self)
{
	ValaCCodeExpression* result;
	ValaCCodeExpression* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_condition;
	result = _tmp0_;
	return result;
}

static gpointer
_vala_ccode_node_ref0 (gpointer self)
{
	return self ? vala_ccode_node_ref (self) : NULL;
}

void
vala_ccode_if_statement_set_condition (ValaCCodeIfStatement* self,
                                       ValaCCodeExpression* value)
{
	ValaCCodeExpression* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _vala_ccode_node_ref0 (value);
	_vala_ccode_node_unref0 (self->priv->_condition);
	self->priv->_condition = _tmp0_;
}

ValaCCodeStatement*
vala_ccode_if_statement_get_true_statement (ValaCCodeIfStatement* self)
{
	ValaCCodeStatement* result;
	ValaCCodeStatement* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_true_statement;
	result = _tmp0_;
	return result;
}

void
vala_ccode_if_statement_set_true_statement (ValaCCodeIfStatement* self,
                                            ValaCCodeStatement* value)
{
	ValaCCodeStatement* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _vala_ccode_node_ref0 (value);
	_vala_ccode_node_unref0 (self->priv->_true_statement);
	self->priv->_true_statement = _tmp0_;
}

ValaCCodeStatement*
vala_ccode_if_statement_get_false_statement (ValaCCodeIfStatement* self)
{
	ValaCCodeStatement* result;
	ValaCCodeStatement* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_false_statement;
	result = _tmp0_;
	return result;
}

void
vala_ccode_if_statement_set_false_statement (ValaCCodeIfStatement* self,
                                             ValaCCodeStatement* value)
{
	ValaCCodeStatement* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _vala_ccode_node_ref0 (value);
	_vala_ccode_node_unref0 (self->priv->_false_statement);
	self->priv->_false_statement = _tmp0_;
}

ValaCCodeIfStatement*
vala_ccode_if_statement_construct (GType object_type,
                                   ValaCCodeExpression* cond,
                                   ValaCCodeStatement* true_stmt,
                                   ValaCCodeStatement* false_stmt)
{
	ValaCCodeIfStatement* self = NULL;
	g_return_val_if_fail (cond != NULL, NULL);
	g_return_val_if_fail (true_stmt != NULL, NULL);
	self = (ValaCCodeIfStatement*) vala_ccode_statement_construct (object_type);
	vala_ccode_if_statement_set_condition (self, cond);
	vala_ccode_if_statement_set_true_statement (self, true_stmt);
	vala_ccode_if_statement_set_false_statement (self, false_stmt);
	return self;
}

ValaCCodeIfStatement*
vala_ccode_if_statement_new (ValaCCodeExpression* cond,
                             ValaCCodeStatement* true_stmt,
                             ValaCCodeStatement* false_stmt)
{
	return vala_ccode_if_statement_construct (VALA_TYPE_CCODE_IF_STATEMENT, cond, true_stmt, false_stmt);
}

gboolean
vala_ccode_if_statement_get_else_if (ValaCCodeIfStatement* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_else_if;
	return result;
}

void
vala_ccode_if_statement_set_else_if (ValaCCodeIfStatement* self,
                                     gboolean value)
{
	g_return_if_fail (self != NULL);
	self->priv->_else_if = value;
}

static void
vala_ccode_if_statement_real_write (ValaCCodeNode* base,
                                    ValaCCodeWriter* writer)
{
	ValaCCodeIfStatement * self;
	gboolean _tmp0_;
	ValaCCodeExpression* _tmp3_;
	gboolean _tmp5_ = FALSE;
	ValaCCodeStatement* _tmp6_;
	ValaCCodeStatement* _tmp11_;
	ValaCCodeStatement* _tmp12_;
	self = (ValaCCodeIfStatement*) base;
	g_return_if_fail (writer != NULL);
	_tmp0_ = self->priv->_else_if;
	if (!_tmp0_) {
		ValaCCodeLineDirective* _tmp1_;
		ValaCCodeLineDirective* _tmp2_;
		_tmp1_ = vala_ccode_node_get_line ((ValaCCodeNode*) self);
		_tmp2_ = _tmp1_;
		vala_ccode_writer_write_indent (writer, _tmp2_);
	} else {
		vala_ccode_writer_write_string (writer, " ");
	}
	vala_ccode_writer_write_string (writer, "if (");
	_tmp3_ = self->priv->_condition;
	if (_tmp3_ != NULL) {
		ValaCCodeExpression* _tmp4_;
		_tmp4_ = self->priv->_condition;
		vala_ccode_node_write ((ValaCCodeNode*) _tmp4_, writer);
	}
	vala_ccode_writer_write_string (writer, ")");
	_tmp6_ = self->priv->_false_statement;
	if (_tmp6_ != NULL) {
		ValaCCodeStatement* _tmp7_;
		_tmp7_ = self->priv->_true_statement;
		_tmp5_ = VALA_IS_CCODE_BLOCK (_tmp7_);
	} else {
		_tmp5_ = FALSE;
	}
	if (_tmp5_) {
		ValaCCodeBlock* cblock = NULL;
		ValaCCodeStatement* _tmp8_;
		ValaCCodeBlock* _tmp9_;
		ValaCCodeBlock* _tmp10_;
		_tmp8_ = self->priv->_true_statement;
		_tmp9_ = _vala_ccode_node_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp8_, VALA_TYPE_CCODE_BLOCK, ValaCCodeBlock));
		cblock = _tmp9_;
		_tmp10_ = cblock;
		vala_ccode_block_set_suppress_newline (_tmp10_, TRUE);
		_vala_ccode_node_unref0 (cblock);
	}
	_tmp11_ = self->priv->_true_statement;
	vala_ccode_node_write ((ValaCCodeNode*) _tmp11_, writer);
	_tmp12_ = self->priv->_false_statement;
	if (_tmp12_ != NULL) {
		gboolean _tmp13_;
		gboolean _tmp14_;
		ValaCCodeStatement* _tmp15_;
		ValaCCodeStatement* _tmp19_;
		_tmp13_ = vala_ccode_writer_get_bol (writer);
		_tmp14_ = _tmp13_;
		if (_tmp14_) {
			vala_ccode_writer_write_indent (writer, NULL);
			vala_ccode_writer_write_string (writer, "else");
		} else {
			vala_ccode_writer_write_string (writer, " else");
		}
		_tmp15_ = self->priv->_false_statement;
		if (VALA_IS_CCODE_IF_STATEMENT (_tmp15_)) {
			ValaCCodeIfStatement* cif = NULL;
			ValaCCodeStatement* _tmp16_;
			ValaCCodeIfStatement* _tmp17_;
			ValaCCodeIfStatement* _tmp18_;
			_tmp16_ = self->priv->_false_statement;
			_tmp17_ = _vala_ccode_node_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp16_, VALA_TYPE_CCODE_IF_STATEMENT, ValaCCodeIfStatement));
			cif = _tmp17_;
			_tmp18_ = cif;
			vala_ccode_if_statement_set_else_if (_tmp18_, TRUE);
			_vala_ccode_node_unref0 (cif);
		}
		_tmp19_ = self->priv->_false_statement;
		vala_ccode_node_write ((ValaCCodeNode*) _tmp19_, writer);
	}
}

static void
vala_ccode_if_statement_class_init (ValaCCodeIfStatementClass * klass,
                                    gpointer klass_data)
{
	vala_ccode_if_statement_parent_class = g_type_class_peek_parent (klass);
	((ValaCCodeNodeClass *) klass)->finalize = vala_ccode_if_statement_finalize;
	g_type_class_adjust_private_offset (klass, &ValaCCodeIfStatement_private_offset);
	((ValaCCodeNodeClass *) klass)->write = (void (*) (ValaCCodeNode*, ValaCCodeWriter*)) vala_ccode_if_statement_real_write;
}

static void
vala_ccode_if_statement_instance_init (ValaCCodeIfStatement * self,
                                       gpointer klass)
{
	self->priv = vala_ccode_if_statement_get_instance_private (self);
}

static void
vala_ccode_if_statement_finalize (ValaCCodeNode * obj)
{
	ValaCCodeIfStatement * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_CCODE_IF_STATEMENT, ValaCCodeIfStatement);
	_vala_ccode_node_unref0 (self->priv->_condition);
	_vala_ccode_node_unref0 (self->priv->_true_statement);
	_vala_ccode_node_unref0 (self->priv->_false_statement);
	VALA_CCODE_NODE_CLASS (vala_ccode_if_statement_parent_class)->finalize (obj);
}

/**
 * Represents an if selection statement in the C code.
 */
static GType
vala_ccode_if_statement_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeIfStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_if_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeIfStatement), 0, (GInstanceInitFunc) vala_ccode_if_statement_instance_init, NULL };
	GType vala_ccode_if_statement_type_id;
	vala_ccode_if_statement_type_id = g_type_register_static (VALA_TYPE_CCODE_STATEMENT, "ValaCCodeIfStatement", &g_define_type_info, 0);
	ValaCCodeIfStatement_private_offset = g_type_add_instance_private (vala_ccode_if_statement_type_id, sizeof (ValaCCodeIfStatementPrivate));
	return vala_ccode_if_statement_type_id;
}

GType
vala_ccode_if_statement_get_type (void)
{
	static volatile gsize vala_ccode_if_statement_type_id__once = 0;
	if (g_once_init_enter (&vala_ccode_if_statement_type_id__once)) {
		GType vala_ccode_if_statement_type_id;
		vala_ccode_if_statement_type_id = vala_ccode_if_statement_get_type_once ();
		g_once_init_leave (&vala_ccode_if_statement_type_id__once, vala_ccode_if_statement_type_id);
	}
	return vala_ccode_if_statement_type_id__once;
}

