/* valastatement.c generated by valac, the Vala compiler
 * generated from valastatement.vala, do not modify */

/* valastatement.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "vala.h"

static GType vala_statement_get_type_once (void);

static void
vala_statement_default_init (ValaStatementIface * iface,
                             gpointer iface_data)
{
}

/**
 * Interface for all statement types.
 */
static GType
vala_statement_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaStatementIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_statement_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType vala_statement_type_id;
	vala_statement_type_id = g_type_register_static (G_TYPE_INTERFACE, "ValaStatement", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (vala_statement_type_id, VALA_TYPE_CODE_NODE);
	return vala_statement_type_id;
}

GType
vala_statement_get_type (void)
{
	static volatile gsize vala_statement_type_id__once = 0;
	if (g_once_init_enter (&vala_statement_type_id__once)) {
		GType vala_statement_type_id;
		vala_statement_type_id = vala_statement_get_type_once ();
		g_once_init_leave (&vala_statement_type_id__once, vala_statement_type_id);
	}
	return vala_statement_type_id__once;
}

