/* valaclassregisterfunction.c generated by valac, the Vala compiler
 * generated from valaclassregisterfunction.vala, do not modify */

/* valaclassregisterfunction.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valacodegen.h"
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <valaccode.h>
#include <valagee.h>
#include <glib-object.h>

#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_code_context_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_context_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))

struct _ValaClassRegisterFunctionPrivate {
	ValaClass* _class_reference;
};

static gint ValaClassRegisterFunction_private_offset;
static gpointer vala_class_register_function_parent_class = NULL;

static ValaTypeSymbol* vala_class_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_base_init_func_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_class_finalize_func_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_base_finalize_func_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_type_flags (ValaTypeRegisterFunction* base);
static ValaSymbolAccessibility vala_class_register_function_real_get_accessibility (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_gtype_value_table_init_function_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_gtype_value_table_free_function_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_gtype_value_table_copy_function_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_gtype_value_table_peek_pointer_function_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_gtype_value_table_collect_value_function_name (ValaTypeRegisterFunction* base);
static gchar* vala_class_register_function_real_get_gtype_value_table_lcopy_value_function_name (ValaTypeRegisterFunction* base);
static ValaCCodeFragment* vala_class_register_function_real_get_type_interface_init_declaration (ValaTypeRegisterFunction* base);
static void vala_class_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base,
                                                                           ValaCodeContext* context,
                                                                           ValaCCodeBlock* block,
                                                                           gboolean plugin);
static void vala_class_register_function_finalize (ValaTypeRegisterFunction * obj);
static GType vala_class_register_function_get_type_once (void);

static inline gpointer
vala_class_register_function_get_instance_private (ValaClassRegisterFunction* self)
{
	return G_STRUCT_MEMBER_P (self, ValaClassRegisterFunction_private_offset);
}

ValaClass*
vala_class_register_function_get_class_reference (ValaClassRegisterFunction* self)
{
	ValaClass* result;
	ValaClass* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_class_reference;
	result = _tmp0_;
	return result;
}

void
vala_class_register_function_set_class_reference (ValaClassRegisterFunction* self,
                                                  ValaClass* value)
{
	g_return_if_fail (self != NULL);
	self->priv->_class_reference = value;
}

/**
 * Creates a new C function to register the specified class at runtime.
 *
 * @param cl a class
 * @return   newly created class register function
 */
ValaClassRegisterFunction*
vala_class_register_function_construct (GType object_type,
                                        ValaClass* cl)
{
	ValaClassRegisterFunction* self = NULL;
	g_return_val_if_fail (cl != NULL, NULL);
	self = (ValaClassRegisterFunction*) vala_typeregister_function_construct (object_type);
	vala_class_register_function_set_class_reference (self, cl);
	return self;
}

ValaClassRegisterFunction*
vala_class_register_function_new (ValaClass* cl)
{
	return vala_class_register_function_construct (VALA_TYPE_CLASS_REGISTER_FUNCTION, cl);
}

static gpointer
_vala_code_node_ref0 (gpointer self)
{
	return self ? vala_code_node_ref (self) : NULL;
}

static ValaTypeSymbol*
vala_class_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	ValaTypeSymbol* _tmp1_;
	ValaTypeSymbol* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = _vala_code_node_ref0 ((ValaTypeSymbol*) _tmp0_);
	result = _tmp1_;
	return result;
}

static gchar*
vala_class_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	gchar* _tmp1_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_get_ccode_type_name ((ValaObjectTypeSymbol*) _tmp0_);
	result = _tmp1_;
	return result;
}

static gchar*
vala_class_register_function_real_get_base_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	ValaConstructor* _tmp1_;
	ValaConstructor* _tmp2_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_class_get_class_constructor (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_ != NULL) {
		ValaClass* _tmp3_;
		gchar* _tmp4_;
		gchar* _tmp5_;
		gchar* _tmp6_;
		gchar* _tmp7_;
		_tmp3_ = self->priv->_class_reference;
		_tmp4_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp3_, NULL);
		_tmp5_ = _tmp4_;
		_tmp6_ = g_strdup_printf ("%s_base_init", _tmp5_);
		_tmp7_ = _tmp6_;
		_g_free0 (_tmp5_);
		result = _tmp7_;
		return result;
	} else {
		gchar* _tmp8_;
		_tmp8_ = g_strdup ("NULL");
		result = _tmp8_;
		return result;
	}
}

static gchar*
vala_class_register_function_real_get_class_finalize_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	ValaDestructor* _tmp1_;
	ValaDestructor* _tmp2_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_class_get_static_destructor (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_ != NULL) {
		ValaClass* _tmp3_;
		gchar* _tmp4_;
		gchar* _tmp5_;
		gchar* _tmp6_;
		gchar* _tmp7_;
		_tmp3_ = self->priv->_class_reference;
		_tmp4_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp3_, NULL);
		_tmp5_ = _tmp4_;
		_tmp6_ = g_strdup_printf ("%s_class_finalize", _tmp5_);
		_tmp7_ = _tmp6_;
		_g_free0 (_tmp5_);
		result = _tmp7_;
		return result;
	} else {
		gchar* _tmp8_;
		_tmp8_ = g_strdup ("NULL");
		result = _tmp8_;
		return result;
	}
}

static gchar*
vala_class_register_function_real_get_base_finalize_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	ValaDestructor* _tmp1_;
	ValaDestructor* _tmp2_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_class_get_class_destructor (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_ != NULL) {
		ValaClass* _tmp3_;
		gchar* _tmp4_;
		gchar* _tmp5_;
		gchar* _tmp6_;
		gchar* _tmp7_;
		_tmp3_ = self->priv->_class_reference;
		_tmp4_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp3_, NULL);
		_tmp5_ = _tmp4_;
		_tmp6_ = g_strdup_printf ("%s_base_finalize", _tmp5_);
		_tmp7_ = _tmp6_;
		_g_free0 (_tmp5_);
		result = _tmp7_;
		return result;
	} else {
		gchar* _tmp8_;
		_tmp8_ = g_strdup ("NULL");
		result = _tmp8_;
		return result;
	}
}

static gchar*
vala_class_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp0_, NULL);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strdup_printf ("%s_class_init", _tmp2_);
	_tmp4_ = _tmp3_;
	_g_free0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static gchar*
vala_class_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_get_ccode_name ((ValaCodeNode*) _tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strdup_printf ("sizeof (%s)", _tmp2_);
	_tmp4_ = _tmp3_;
	_g_free0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static gchar*
vala_class_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp0_, NULL);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strdup_printf ("%s_instance_init", _tmp2_);
	_tmp4_ = _tmp3_;
	_g_free0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static gchar*
vala_class_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	ValaClass* _tmp1_;
	ValaClass* _tmp2_;
	gchar* _tmp3_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_class_get_base_class (_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = vala_get_ccode_type_id ((ValaCodeNode*) _tmp2_);
	result = _tmp3_;
	return result;
}

static gchar*
vala_class_register_function_real_get_type_flags (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_class_get_is_abstract (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_) {
		gchar* _tmp3_;
		_tmp3_ = g_strdup ("G_TYPE_FLAG_ABSTRACT");
		result = _tmp3_;
		return result;
	} else {
		gboolean _tmp4_ = FALSE;
		ValaCodeContext* _tmp5_;
		ValaCodeContext* _tmp6_;
		gboolean _tmp7_;
		_tmp5_ = vala_code_context_get ();
		_tmp6_ = _tmp5_;
		_tmp7_ = vala_code_context_require_glib_version (_tmp6_, 2, 70);
		_vala_code_context_unref0 (_tmp6_);
		if (_tmp7_) {
			ValaClass* _tmp8_;
			gboolean _tmp9_;
			gboolean _tmp10_;
			_tmp8_ = self->priv->_class_reference;
			_tmp9_ = vala_class_get_is_sealed (_tmp8_);
			_tmp10_ = _tmp9_;
			_tmp4_ = _tmp10_;
		} else {
			_tmp4_ = FALSE;
		}
		if (_tmp4_) {
			gchar* _tmp11_;
			_tmp11_ = g_strdup ("G_TYPE_FLAG_FINAL");
			result = _tmp11_;
			return result;
		} else {
			gchar* _tmp12_;
			_tmp12_ = g_strdup ("0");
			result = _tmp12_;
			return result;
		}
	}
}

static ValaSymbolAccessibility
vala_class_register_function_real_get_accessibility (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaClass* _tmp0_;
	ValaSymbolAccessibility _tmp1_;
	ValaSymbolAccessibility _tmp2_;
	ValaSymbolAccessibility result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = self->priv->_class_reference;
	_tmp1_ = vala_symbol_get_access ((ValaSymbol*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static gchar*
vala_class_register_function_real_get_gtype_value_table_init_function_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	gboolean _tmp0_ = FALSE;
	ValaClass* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean is_fundamental = FALSE;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp1_ = self->priv->_class_reference;
	_tmp2_ = vala_class_get_is_compact (_tmp1_);
	_tmp3_ = _tmp2_;
	if (!_tmp3_) {
		ValaClass* _tmp4_;
		ValaClass* _tmp5_;
		ValaClass* _tmp6_;
		_tmp4_ = self->priv->_class_reference;
		_tmp5_ = vala_class_get_base_class (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_ == NULL;
	} else {
		_tmp0_ = FALSE;
	}
	is_fundamental = _tmp0_;
	if (is_fundamental) {
		ValaClass* _tmp7_;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		_tmp7_ = self->priv->_class_reference;
		_tmp8_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp7_, "value_");
		_tmp9_ = _tmp8_;
		_tmp10_ = g_strdup_printf ("%s_init", _tmp9_);
		_tmp11_ = _tmp10_;
		_g_free0 (_tmp9_);
		result = _tmp11_;
		return result;
	}
	result = NULL;
	return result;
}

static gchar*
vala_class_register_function_real_get_gtype_value_table_free_function_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	gboolean _tmp0_ = FALSE;
	ValaClass* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean is_fundamental = FALSE;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp1_ = self->priv->_class_reference;
	_tmp2_ = vala_class_get_is_compact (_tmp1_);
	_tmp3_ = _tmp2_;
	if (!_tmp3_) {
		ValaClass* _tmp4_;
		ValaClass* _tmp5_;
		ValaClass* _tmp6_;
		_tmp4_ = self->priv->_class_reference;
		_tmp5_ = vala_class_get_base_class (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_ == NULL;
	} else {
		_tmp0_ = FALSE;
	}
	is_fundamental = _tmp0_;
	if (is_fundamental) {
		ValaClass* _tmp7_;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		_tmp7_ = self->priv->_class_reference;
		_tmp8_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp7_, "value_");
		_tmp9_ = _tmp8_;
		_tmp10_ = g_strdup_printf ("%s_free_value", _tmp9_);
		_tmp11_ = _tmp10_;
		_g_free0 (_tmp9_);
		result = _tmp11_;
		return result;
	}
	result = NULL;
	return result;
}

static gchar*
vala_class_register_function_real_get_gtype_value_table_copy_function_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	gboolean _tmp0_ = FALSE;
	ValaClass* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean is_fundamental = FALSE;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp1_ = self->priv->_class_reference;
	_tmp2_ = vala_class_get_is_compact (_tmp1_);
	_tmp3_ = _tmp2_;
	if (!_tmp3_) {
		ValaClass* _tmp4_;
		ValaClass* _tmp5_;
		ValaClass* _tmp6_;
		_tmp4_ = self->priv->_class_reference;
		_tmp5_ = vala_class_get_base_class (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_ == NULL;
	} else {
		_tmp0_ = FALSE;
	}
	is_fundamental = _tmp0_;
	if (is_fundamental) {
		ValaClass* _tmp7_;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		_tmp7_ = self->priv->_class_reference;
		_tmp8_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp7_, "value_");
		_tmp9_ = _tmp8_;
		_tmp10_ = g_strdup_printf ("%s_copy_value", _tmp9_);
		_tmp11_ = _tmp10_;
		_g_free0 (_tmp9_);
		result = _tmp11_;
		return result;
	}
	result = NULL;
	return result;
}

static gchar*
vala_class_register_function_real_get_gtype_value_table_peek_pointer_function_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	gboolean _tmp0_ = FALSE;
	ValaClass* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean is_fundamental = FALSE;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp1_ = self->priv->_class_reference;
	_tmp2_ = vala_class_get_is_compact (_tmp1_);
	_tmp3_ = _tmp2_;
	if (!_tmp3_) {
		ValaClass* _tmp4_;
		ValaClass* _tmp5_;
		ValaClass* _tmp6_;
		_tmp4_ = self->priv->_class_reference;
		_tmp5_ = vala_class_get_base_class (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_ == NULL;
	} else {
		_tmp0_ = FALSE;
	}
	is_fundamental = _tmp0_;
	if (is_fundamental) {
		ValaClass* _tmp7_;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		_tmp7_ = self->priv->_class_reference;
		_tmp8_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp7_, "value_");
		_tmp9_ = _tmp8_;
		_tmp10_ = g_strdup_printf ("%s_peek_pointer", _tmp9_);
		_tmp11_ = _tmp10_;
		_g_free0 (_tmp9_);
		result = _tmp11_;
		return result;
	}
	result = NULL;
	return result;
}

static gchar*
vala_class_register_function_real_get_gtype_value_table_collect_value_function_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	gboolean _tmp0_ = FALSE;
	ValaClass* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean is_fundamental = FALSE;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp1_ = self->priv->_class_reference;
	_tmp2_ = vala_class_get_is_compact (_tmp1_);
	_tmp3_ = _tmp2_;
	if (!_tmp3_) {
		ValaClass* _tmp4_;
		ValaClass* _tmp5_;
		ValaClass* _tmp6_;
		_tmp4_ = self->priv->_class_reference;
		_tmp5_ = vala_class_get_base_class (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_ == NULL;
	} else {
		_tmp0_ = FALSE;
	}
	is_fundamental = _tmp0_;
	if (is_fundamental) {
		ValaClass* _tmp7_;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		_tmp7_ = self->priv->_class_reference;
		_tmp8_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp7_, "value_");
		_tmp9_ = _tmp8_;
		_tmp10_ = g_strdup_printf ("%s_collect_value", _tmp9_);
		_tmp11_ = _tmp10_;
		_g_free0 (_tmp9_);
		result = _tmp11_;
		return result;
	}
	result = NULL;
	return result;
}

static gchar*
vala_class_register_function_real_get_gtype_value_table_lcopy_value_function_name (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	gboolean _tmp0_ = FALSE;
	ValaClass* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean is_fundamental = FALSE;
	gchar* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp1_ = self->priv->_class_reference;
	_tmp2_ = vala_class_get_is_compact (_tmp1_);
	_tmp3_ = _tmp2_;
	if (!_tmp3_) {
		ValaClass* _tmp4_;
		ValaClass* _tmp5_;
		ValaClass* _tmp6_;
		_tmp4_ = self->priv->_class_reference;
		_tmp5_ = vala_class_get_base_class (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp0_ = _tmp6_ == NULL;
	} else {
		_tmp0_ = FALSE;
	}
	is_fundamental = _tmp0_;
	if (is_fundamental) {
		ValaClass* _tmp7_;
		gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		_tmp7_ = self->priv->_class_reference;
		_tmp8_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp7_, "value_");
		_tmp9_ = _tmp8_;
		_tmp10_ = g_strdup_printf ("%s_lcopy_value", _tmp9_);
		_tmp11_ = _tmp10_;
		_g_free0 (_tmp9_);
		result = _tmp11_;
		return result;
	}
	result = NULL;
	return result;
}

static ValaCCodeFragment*
vala_class_register_function_real_get_type_interface_init_declaration (ValaTypeRegisterFunction* base)
{
	ValaClassRegisterFunction * self;
	ValaCCodeFragment* frag = NULL;
	ValaCCodeFragment* _tmp0_;
	ValaCCodeFragment* result;
	self = (ValaClassRegisterFunction*) base;
	_tmp0_ = vala_ccode_fragment_new ();
	frag = _tmp0_;
	{
		ValaList* _base_type_list = NULL;
		ValaClass* _tmp1_;
		ValaList* _tmp2_;
		gint _base_type_size = 0;
		ValaList* _tmp3_;
		gint _tmp4_;
		gint _tmp5_;
		gint _base_type_index = 0;
		_tmp1_ = self->priv->_class_reference;
		_tmp2_ = vala_class_get_base_types (_tmp1_);
		_base_type_list = _tmp2_;
		_tmp3_ = _base_type_list;
		_tmp4_ = vala_collection_get_size ((ValaCollection*) _tmp3_);
		_tmp5_ = _tmp4_;
		_base_type_size = _tmp5_;
		_base_type_index = -1;
		while (TRUE) {
			gint _tmp6_;
			gint _tmp7_;
			ValaDataType* base_type = NULL;
			ValaList* _tmp8_;
			gpointer _tmp9_;
			ValaDataType* _tmp10_;
			ValaTypeSymbol* _tmp11_;
			ValaTypeSymbol* _tmp12_;
			ValaInterface* iface = NULL;
			ValaDataType* _tmp13_;
			ValaTypeSymbol* _tmp14_;
			ValaTypeSymbol* _tmp15_;
			gchar* iface_info_name = NULL;
			ValaInterface* _tmp16_;
			gchar* _tmp17_;
			gchar* _tmp18_;
			gchar* _tmp19_;
			gchar* _tmp20_;
			ValaCCodeDeclaration* ctypedecl = NULL;
			ValaCCodeDeclaration* _tmp21_;
			ValaCCodeDeclaration* _tmp22_;
			ValaCCodeDeclaration* _tmp23_;
			const gchar* _tmp24_;
			ValaClass* _tmp25_;
			gchar* _tmp26_;
			gchar* _tmp27_;
			ValaInterface* _tmp28_;
			gchar* _tmp29_;
			gchar* _tmp30_;
			gchar* _tmp31_;
			gchar* _tmp32_;
			ValaCCodeConstant* _tmp33_;
			ValaCCodeConstant* _tmp34_;
			ValaCCodeVariableDeclarator* _tmp35_;
			ValaCCodeVariableDeclarator* _tmp36_;
			ValaCCodeFragment* _tmp37_;
			ValaCCodeDeclaration* _tmp38_;
			_base_type_index = _base_type_index + 1;
			_tmp6_ = _base_type_index;
			_tmp7_ = _base_type_size;
			if (!(_tmp6_ < _tmp7_)) {
				break;
			}
			_tmp8_ = _base_type_list;
			_tmp9_ = vala_list_get (_tmp8_, _base_type_index);
			base_type = (ValaDataType*) _tmp9_;
			_tmp10_ = base_type;
			_tmp11_ = vala_data_type_get_type_symbol (_tmp10_);
			_tmp12_ = _tmp11_;
			if (!G_TYPE_CHECK_INSTANCE_TYPE (_tmp12_, VALA_TYPE_INTERFACE)) {
				_vala_code_node_unref0 (base_type);
				continue;
			}
			_tmp13_ = base_type;
			_tmp14_ = vala_data_type_get_type_symbol (_tmp13_);
			_tmp15_ = _tmp14_;
			iface = G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, VALA_TYPE_INTERFACE, ValaInterface);
			_tmp16_ = iface;
			_tmp17_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp16_, NULL);
			_tmp18_ = _tmp17_;
			_tmp19_ = g_strdup_printf ("%s_info", _tmp18_);
			_tmp20_ = _tmp19_;
			_g_free0 (_tmp18_);
			iface_info_name = _tmp20_;
			_tmp21_ = vala_ccode_declaration_new ("const GInterfaceInfo");
			ctypedecl = _tmp21_;
			_tmp22_ = ctypedecl;
			vala_ccode_node_set_modifiers ((ValaCCodeNode*) _tmp22_, VALA_CCODE_MODIFIERS_STATIC);
			_tmp23_ = ctypedecl;
			_tmp24_ = iface_info_name;
			_tmp25_ = self->priv->_class_reference;
			_tmp26_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp25_, NULL);
			_tmp27_ = _tmp26_;
			_tmp28_ = iface;
			_tmp29_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp28_, NULL);
			_tmp30_ = _tmp29_;
			_tmp31_ = g_strdup_printf ("{ (GInterfaceInitFunc) %s_%s_interface_init, (GInterfaceFinalizeFunc) " \
"NULL, NULL}", _tmp27_, _tmp30_);
			_tmp32_ = _tmp31_;
			_tmp33_ = vala_ccode_constant_new (_tmp32_);
			_tmp34_ = _tmp33_;
			_tmp35_ = vala_ccode_variable_declarator_new (_tmp24_, (ValaCCodeExpression*) _tmp34_, NULL);
			_tmp36_ = _tmp35_;
			vala_ccode_declaration_add_declarator (_tmp23_, (ValaCCodeDeclarator*) _tmp36_);
			_vala_ccode_node_unref0 (_tmp36_);
			_vala_ccode_node_unref0 (_tmp34_);
			_g_free0 (_tmp32_);
			_g_free0 (_tmp30_);
			_g_free0 (_tmp27_);
			_tmp37_ = frag;
			_tmp38_ = ctypedecl;
			vala_ccode_fragment_append (_tmp37_, (ValaCCodeNode*) _tmp38_);
			_vala_ccode_node_unref0 (ctypedecl);
			_g_free0 (iface_info_name);
			_vala_code_node_unref0 (base_type);
		}
	}
	result = frag;
	return result;
}

static void
vala_class_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base,
                                                                      ValaCodeContext* context,
                                                                      ValaCCodeBlock* block,
                                                                      gboolean plugin)
{
	ValaClassRegisterFunction * self;
	ValaCodeGenerator* _tmp77_;
	ValaCodeGenerator* _tmp78_;
	ValaClass* _tmp79_;
	self = (ValaClassRegisterFunction*) base;
	g_return_if_fail (context != NULL);
	g_return_if_fail (block != NULL);
	{
		ValaList* _base_type_list = NULL;
		ValaClass* _tmp0_;
		ValaList* _tmp1_;
		gint _base_type_size = 0;
		ValaList* _tmp2_;
		gint _tmp3_;
		gint _tmp4_;
		gint _base_type_index = 0;
		_tmp0_ = self->priv->_class_reference;
		_tmp1_ = vala_class_get_base_types (_tmp0_);
		_base_type_list = _tmp1_;
		_tmp2_ = _base_type_list;
		_tmp3_ = vala_collection_get_size ((ValaCollection*) _tmp2_);
		_tmp4_ = _tmp3_;
		_base_type_size = _tmp4_;
		_base_type_index = -1;
		while (TRUE) {
			gint _tmp5_;
			gint _tmp6_;
			ValaDataType* base_type = NULL;
			ValaList* _tmp7_;
			gpointer _tmp8_;
			ValaDataType* _tmp9_;
			ValaTypeSymbol* _tmp10_;
			ValaTypeSymbol* _tmp11_;
			ValaInterface* iface = NULL;
			ValaDataType* _tmp12_;
			ValaTypeSymbol* _tmp13_;
			ValaTypeSymbol* _tmp14_;
			gchar* iface_info_name = NULL;
			ValaInterface* _tmp15_;
			gchar* _tmp16_;
			gchar* _tmp17_;
			gchar* _tmp18_;
			gchar* _tmp19_;
			_base_type_index = _base_type_index + 1;
			_tmp5_ = _base_type_index;
			_tmp6_ = _base_type_size;
			if (!(_tmp5_ < _tmp6_)) {
				break;
			}
			_tmp7_ = _base_type_list;
			_tmp8_ = vala_list_get (_tmp7_, _base_type_index);
			base_type = (ValaDataType*) _tmp8_;
			_tmp9_ = base_type;
			_tmp10_ = vala_data_type_get_type_symbol (_tmp9_);
			_tmp11_ = _tmp10_;
			if (!G_TYPE_CHECK_INSTANCE_TYPE (_tmp11_, VALA_TYPE_INTERFACE)) {
				_vala_code_node_unref0 (base_type);
				continue;
			}
			_tmp12_ = base_type;
			_tmp13_ = vala_data_type_get_type_symbol (_tmp12_);
			_tmp14_ = _tmp13_;
			iface = G_TYPE_CHECK_INSTANCE_CAST (_tmp14_, VALA_TYPE_INTERFACE, ValaInterface);
			_tmp15_ = iface;
			_tmp16_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp15_, NULL);
			_tmp17_ = _tmp16_;
			_tmp18_ = g_strdup_printf ("%s_info", _tmp17_);
			_tmp19_ = _tmp18_;
			_g_free0 (_tmp17_);
			iface_info_name = _tmp19_;
			if (!plugin) {
				ValaCCodeFunctionCall* reg_call = NULL;
				ValaCCodeIdentifier* _tmp20_;
				ValaCCodeIdentifier* _tmp21_;
				ValaCCodeFunctionCall* _tmp22_;
				ValaCCodeFunctionCall* _tmp23_;
				ValaCCodeFunctionCall* _tmp24_;
				ValaClass* _tmp25_;
				gchar* _tmp26_;
				gchar* _tmp27_;
				gchar* _tmp28_;
				gchar* _tmp29_;
				ValaCCodeIdentifier* _tmp30_;
				ValaCCodeIdentifier* _tmp31_;
				ValaCCodeFunctionCall* _tmp32_;
				ValaInterface* _tmp33_;
				gchar* _tmp34_;
				gchar* _tmp35_;
				ValaCCodeIdentifier* _tmp36_;
				ValaCCodeIdentifier* _tmp37_;
				ValaCCodeFunctionCall* _tmp38_;
				const gchar* _tmp39_;
				gchar* _tmp40_;
				gchar* _tmp41_;
				ValaCCodeIdentifier* _tmp42_;
				ValaCCodeIdentifier* _tmp43_;
				ValaCCodeFunctionCall* _tmp44_;
				ValaCCodeExpressionStatement* _tmp45_;
				ValaCCodeExpressionStatement* _tmp46_;
				_tmp20_ = vala_ccode_identifier_new ("g_type_add_interface_static");
				_tmp21_ = _tmp20_;
				_tmp22_ = vala_ccode_function_call_new ((ValaCCodeExpression*) _tmp21_);
				_tmp23_ = _tmp22_;
				_vala_ccode_node_unref0 (_tmp21_);
				reg_call = _tmp23_;
				_tmp24_ = reg_call;
				_tmp25_ = self->priv->_class_reference;
				_tmp26_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp25_, NULL);
				_tmp27_ = _tmp26_;
				_tmp28_ = g_strdup_printf ("%s_type_id", _tmp27_);
				_tmp29_ = _tmp28_;
				_tmp30_ = vala_ccode_identifier_new (_tmp29_);
				_tmp31_ = _tmp30_;
				vala_ccode_function_call_add_argument (_tmp24_, (ValaCCodeExpression*) _tmp31_);
				_vala_ccode_node_unref0 (_tmp31_);
				_g_free0 (_tmp29_);
				_g_free0 (_tmp27_);
				_tmp32_ = reg_call;
				_tmp33_ = iface;
				_tmp34_ = vala_get_ccode_type_id ((ValaCodeNode*) _tmp33_);
				_tmp35_ = _tmp34_;
				_tmp36_ = vala_ccode_identifier_new (_tmp35_);
				_tmp37_ = _tmp36_;
				vala_ccode_function_call_add_argument (_tmp32_, (ValaCCodeExpression*) _tmp37_);
				_vala_ccode_node_unref0 (_tmp37_);
				_g_free0 (_tmp35_);
				_tmp38_ = reg_call;
				_tmp39_ = iface_info_name;
				_tmp40_ = g_strdup_printf ("&%s", _tmp39_);
				_tmp41_ = _tmp40_;
				_tmp42_ = vala_ccode_identifier_new (_tmp41_);
				_tmp43_ = _tmp42_;
				vala_ccode_function_call_add_argument (_tmp38_, (ValaCCodeExpression*) _tmp43_);
				_vala_ccode_node_unref0 (_tmp43_);
				_g_free0 (_tmp41_);
				_tmp44_ = reg_call;
				_tmp45_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) _tmp44_);
				_tmp46_ = _tmp45_;
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) _tmp46_);
				_vala_ccode_node_unref0 (_tmp46_);
				_vala_ccode_node_unref0 (reg_call);
			} else {
				ValaCCodeFunctionCall* reg_call = NULL;
				ValaCCodeIdentifier* _tmp47_;
				ValaCCodeIdentifier* _tmp48_;
				ValaCCodeFunctionCall* _tmp49_;
				ValaCCodeFunctionCall* _tmp50_;
				ValaCCodeFunctionCall* _tmp51_;
				ValaCCodeIdentifier* _tmp52_;
				ValaCCodeIdentifier* _tmp53_;
				ValaCCodeFunctionCall* _tmp54_;
				ValaClass* _tmp55_;
				gchar* _tmp56_;
				gchar* _tmp57_;
				gchar* _tmp58_;
				gchar* _tmp59_;
				ValaCCodeIdentifier* _tmp60_;
				ValaCCodeIdentifier* _tmp61_;
				ValaCCodeFunctionCall* _tmp62_;
				ValaInterface* _tmp63_;
				gchar* _tmp64_;
				gchar* _tmp65_;
				ValaCCodeIdentifier* _tmp66_;
				ValaCCodeIdentifier* _tmp67_;
				ValaCCodeFunctionCall* _tmp68_;
				const gchar* _tmp69_;
				gchar* _tmp70_;
				gchar* _tmp71_;
				ValaCCodeIdentifier* _tmp72_;
				ValaCCodeIdentifier* _tmp73_;
				ValaCCodeFunctionCall* _tmp74_;
				ValaCCodeExpressionStatement* _tmp75_;
				ValaCCodeExpressionStatement* _tmp76_;
				_tmp47_ = vala_ccode_identifier_new ("g_type_module_add_interface");
				_tmp48_ = _tmp47_;
				_tmp49_ = vala_ccode_function_call_new ((ValaCCodeExpression*) _tmp48_);
				_tmp50_ = _tmp49_;
				_vala_ccode_node_unref0 (_tmp48_);
				reg_call = _tmp50_;
				_tmp51_ = reg_call;
				_tmp52_ = vala_ccode_identifier_new ("module");
				_tmp53_ = _tmp52_;
				vala_ccode_function_call_add_argument (_tmp51_, (ValaCCodeExpression*) _tmp53_);
				_vala_ccode_node_unref0 (_tmp53_);
				_tmp54_ = reg_call;
				_tmp55_ = self->priv->_class_reference;
				_tmp56_ = vala_get_ccode_lower_case_name ((ValaCodeNode*) _tmp55_, NULL);
				_tmp57_ = _tmp56_;
				_tmp58_ = g_strdup_printf ("%s_type_id", _tmp57_);
				_tmp59_ = _tmp58_;
				_tmp60_ = vala_ccode_identifier_new (_tmp59_);
				_tmp61_ = _tmp60_;
				vala_ccode_function_call_add_argument (_tmp54_, (ValaCCodeExpression*) _tmp61_);
				_vala_ccode_node_unref0 (_tmp61_);
				_g_free0 (_tmp59_);
				_g_free0 (_tmp57_);
				_tmp62_ = reg_call;
				_tmp63_ = iface;
				_tmp64_ = vala_get_ccode_type_id ((ValaCodeNode*) _tmp63_);
				_tmp65_ = _tmp64_;
				_tmp66_ = vala_ccode_identifier_new (_tmp65_);
				_tmp67_ = _tmp66_;
				vala_ccode_function_call_add_argument (_tmp62_, (ValaCCodeExpression*) _tmp67_);
				_vala_ccode_node_unref0 (_tmp67_);
				_g_free0 (_tmp65_);
				_tmp68_ = reg_call;
				_tmp69_ = iface_info_name;
				_tmp70_ = g_strdup_printf ("&%s", _tmp69_);
				_tmp71_ = _tmp70_;
				_tmp72_ = vala_ccode_identifier_new (_tmp71_);
				_tmp73_ = _tmp72_;
				vala_ccode_function_call_add_argument (_tmp68_, (ValaCCodeExpression*) _tmp73_);
				_vala_ccode_node_unref0 (_tmp73_);
				_g_free0 (_tmp71_);
				_tmp74_ = reg_call;
				_tmp75_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) _tmp74_);
				_tmp76_ = _tmp75_;
				vala_ccode_block_add_statement (block, (ValaCCodeNode*) _tmp76_);
				_vala_ccode_node_unref0 (_tmp76_);
				_vala_ccode_node_unref0 (reg_call);
			}
			_g_free0 (iface_info_name);
			_vala_code_node_unref0 (base_type);
		}
	}
	_tmp77_ = vala_code_context_get_codegen (context);
	_tmp78_ = _tmp77_;
	_tmp79_ = self->priv->_class_reference;
	vala_ccode_base_module_register_dbus_info (G_TYPE_CHECK_INSTANCE_CAST (_tmp78_, VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule), block, (ValaObjectTypeSymbol*) _tmp79_);
}

static void
vala_class_register_function_class_init (ValaClassRegisterFunctionClass * klass,
                                         gpointer klass_data)
{
	vala_class_register_function_parent_class = g_type_class_peek_parent (klass);
	((ValaTypeRegisterFunctionClass *) klass)->finalize = vala_class_register_function_finalize;
	g_type_class_adjust_private_offset (klass, &ValaClassRegisterFunction_private_offset);
	((ValaTypeRegisterFunctionClass *) klass)->get_type_declaration = (ValaTypeSymbol* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_type_declaration;
	((ValaTypeRegisterFunctionClass *) klass)->get_type_struct_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_type_struct_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_base_init_func_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_base_init_func_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_class_finalize_func_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_class_finalize_func_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_base_finalize_func_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_base_finalize_func_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_class_init_func_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_class_init_func_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_instance_struct_size = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_instance_struct_size;
	((ValaTypeRegisterFunctionClass *) klass)->get_instance_init_func_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_instance_init_func_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_parent_type_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_parent_type_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_type_flags = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_type_flags;
	((ValaTypeRegisterFunctionClass *) klass)->get_accessibility = (ValaSymbolAccessibility (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_accessibility;
	((ValaTypeRegisterFunctionClass *) klass)->get_gtype_value_table_init_function_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_gtype_value_table_init_function_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_gtype_value_table_free_function_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_gtype_value_table_free_function_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_gtype_value_table_copy_function_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_gtype_value_table_copy_function_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_gtype_value_table_peek_pointer_function_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_gtype_value_table_peek_pointer_function_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_gtype_value_table_collect_value_function_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_gtype_value_table_collect_value_function_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_gtype_value_table_lcopy_value_function_name = (gchar* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_gtype_value_table_lcopy_value_function_name;
	((ValaTypeRegisterFunctionClass *) klass)->get_type_interface_init_declaration = (ValaCCodeFragment* (*) (ValaTypeRegisterFunction*)) vala_class_register_function_real_get_type_interface_init_declaration;
	((ValaTypeRegisterFunctionClass *) klass)->get_type_interface_init_statements = (void (*) (ValaTypeRegisterFunction*, ValaCodeContext*, ValaCCodeBlock*, gboolean)) vala_class_register_function_real_get_type_interface_init_statements;
}

static void
vala_class_register_function_instance_init (ValaClassRegisterFunction * self,
                                            gpointer klass)
{
	self->priv = vala_class_register_function_get_instance_private (self);
}

static void
vala_class_register_function_finalize (ValaTypeRegisterFunction * obj)
{
	ValaClassRegisterFunction * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_CLASS_REGISTER_FUNCTION, ValaClassRegisterFunction);
	VALA_TYPEREGISTER_FUNCTION_CLASS (vala_class_register_function_parent_class)->finalize (obj);
}

/**
 * C function to register a class at runtime.
 */
static GType
vala_class_register_function_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaClassRegisterFunctionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_class_register_function_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaClassRegisterFunction), 0, (GInstanceInitFunc) vala_class_register_function_instance_init, NULL };
	GType vala_class_register_function_type_id;
	vala_class_register_function_type_id = g_type_register_static (VALA_TYPE_TYPEREGISTER_FUNCTION, "ValaClassRegisterFunction", &g_define_type_info, 0);
	ValaClassRegisterFunction_private_offset = g_type_add_instance_private (vala_class_register_function_type_id, sizeof (ValaClassRegisterFunctionPrivate));
	return vala_class_register_function_type_id;
}

GType
vala_class_register_function_get_type (void)
{
	static volatile gsize vala_class_register_function_type_id__once = 0;
	if (g_once_init_enter (&vala_class_register_function_type_id__once)) {
		GType vala_class_register_function_type_id;
		vala_class_register_function_type_id = vala_class_register_function_get_type_once ();
		g_once_init_leave (&vala_class_register_function_type_id__once, vala_class_register_function_type_id);
	}
	return vala_class_register_function_type_id__once;
}

