/* valaliteral.c generated by valac, the Vala compiler
 * generated from valaliteral.vala, do not modify */

/* valaliteral.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "vala.h"
#include <glib.h>

static gpointer vala_literal_parent_class = NULL;

static gboolean vala_literal_real_is_constant (ValaExpression* base);
static gboolean vala_literal_real_is_pure (ValaExpression* base);
static GType vala_literal_get_type_once (void);

static gboolean
vala_literal_real_is_constant (ValaExpression* base)
{
	ValaLiteral * self;
	gboolean result;
	self = (ValaLiteral*) base;
	result = TRUE;
	return result;
}

static gboolean
vala_literal_real_is_pure (ValaExpression* base)
{
	ValaLiteral * self;
	gboolean result;
	self = (ValaLiteral*) base;
	result = TRUE;
	return result;
}

ValaLiteral*
vala_literal_construct (GType object_type)
{
	ValaLiteral* self = NULL;
	self = (ValaLiteral*) vala_expression_construct (object_type);
	return self;
}

static void
vala_literal_class_init (ValaLiteralClass * klass,
                         gpointer klass_data)
{
	vala_literal_parent_class = g_type_class_peek_parent (klass);
	((ValaExpressionClass *) klass)->is_constant = (gboolean (*) (ValaExpression*)) vala_literal_real_is_constant;
	((ValaExpressionClass *) klass)->is_pure = (gboolean (*) (ValaExpression*)) vala_literal_real_is_pure;
}

static void
vala_literal_instance_init (ValaLiteral * self,
                            gpointer klass)
{
}

/**
 * Base class for all literals in the source code.
 */
static GType
vala_literal_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaLiteralClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_literal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaLiteral), 0, (GInstanceInitFunc) vala_literal_instance_init, NULL };
	GType vala_literal_type_id;
	vala_literal_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaLiteral", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	return vala_literal_type_id;
}

GType
vala_literal_get_type (void)
{
	static volatile gsize vala_literal_type_id__once = 0;
	if (g_once_init_enter (&vala_literal_type_id__once)) {
		GType vala_literal_type_id;
		vala_literal_type_id = vala_literal_get_type_once ();
		g_once_init_leave (&vala_literal_type_id__once, vala_literal_type_id);
	}
	return vala_literal_type_id__once;
}

